import numpy as np
import xgboost as xgb
from sklearn.base import BaseEstimator, TransformerMixin


class ModelWrapper(BaseEstimator, TransformerMixin):
    """
    Универсальная обертка для стандартизации API вызова модели.
    Используется для возможности исполнения пайплайна с
    вызовом любого типа используемых моделей.
    Поддерживаемые модели: DS-Template, xgboost, catboost,
    lightgbm, sklearn.

    Parameters
    ----------
    estimator: estimator object implementing 'fit'
        Экземпляр модели для применения к набору данных.

    fill_value: int/float, optional, default = None
        Значене для заполнения пропусков в данных.
        Опциональный параметр, по умолчанию, не используется.

    log_scale: bool, optional, default = False
        Флаг, означающий, что прогнозы модели представлены в
        логарифмированной-шкале. После применения модели
        требуется преобразование прогнозов в исходную
        шкалу. Опциональный параметр, по умолчанию,
        не используется. Используется для задачи
        регрессии.

    log_bias: int/float, optional, default = 0
        Смещение, используемое в аргументе логарифма.
        Пример: если целевая переменная при обучении
        преобразовывалась с помощь np.log(target + 1),
        то log_bias = 1. Опциональный параметр, по умолчанию,
        равен 0. Используется для задачи регрессии.

    min_target: int/float, optional, default = 0
        Минимальное значение целевой переменной в
        обучающей выборке. Опциональный параметр, по
        умолчанию, равен 0. Используется для задачи регрессии.

    """
    def __init__(self,
                 estimator,
                 fill_value=None,
                 log_scale=False,
                 log_bias=0,
                 min_target=0) -> None:

        self.estimator = estimator
        self.fill_value = fill_value
        self.log_scale = log_scale
        self.log_bias = log_bias
        self.min_target = min_target

    def transform(self, X, y=None) -> np.array:
        """
        Применение модели к новому набору данных.
        Parameters
        ----------
        X: pandas.DataFrame, shape = [n_samples, n_features]
            Матрица признаков для применения модели.
        Returns
        -------
        y_pred: numpy.array
            Вектор прогнозов.

        """
        method = self._choose_predict_method()
        if self.fill_value:
            X = X.fillna(self.fill_value)

        y_pred = method(X)
        return self.postprocessing(y_pred)

    def _choose_predict_method(self) -> callable:
        """
        Выбор метода для прогнозирования.
        """
        module = self.estimator.__module__
        predict_proba_flg = hasattr(self.estimator, "predict_proba")

        if "dspl.models" in module:
            return self.estimator.transform
        elif "xgboost.core" in module:
            return self.xgb_booster_predict
        elif predict_proba_flg:
            return self.estimator.predict_proba
        else:
            return self.estimator.predict

    def postprocessing(self, y_pred: np.array) -> np.array:
        """
        Постпроцессинг полученных прогнозов.
        Если модель возвращает матрицу прогнозов с двумя
        столбцами - то возвращается вероятность принадлежности
        объекта в целевому классу (второй столбец).
        Если при обучении модели осуществлялось логарифмирование
        таргета - то возвращается прогноз в исходном диапазоне
        целевой переменной.
        Parameters
        ----------
        y_pred: np.array, shape = [n_samples, ]
            Вектор прогнозов.
        Returns
        -------
        y_pred_transformed: np.array, shape = [n_samples, ]
            Преобразованный вектор прогнозов.

        """
        shape = np.ndim(y_pred)

        if shape == 2:
            return y_pred[:, 1]
        if self.log_scale:
            return np.exp(y_pred) - self.log_bias + self.min_target

        return y_pred

    def xgb_booster_predict(self, X) -> np.array:
        """
        Применение нативной модели xgboost к новому
        набору данных X. Для применения нативной версии
        xgboost требуется преобразования набора данных в
        xgboost.DMatrix.
        Parameters
        ----------
        X: pandas.DataFrame, shape = [n_samples, n_features]
            Матрица признаков для применения модели.
        Returns
        -------
        y_pred: numpy.array
            Вектор прогнозов.

        """
        X = xgb.DMatrix(X)
        return self.estimator.predict(X)



from setuptools import setup, find_packages

requirements = [
    'astroid==2.3.3',
    'attrs==19.3.0',
    'backcall==0.1.0',
    'bayesian-optimization==1.0.1',
    'bcrypt==3.1.7',
    'bleach==3.1.4',
    'catboost==0.18.1',
    'cffi==1.13.2',
    'cloudpickle==1.2.1',
    'cryptography==2.8',
    'cycler==0.10.0',
    'dask==2.3.0',
    'decorator==4.4.1',
    'defusedxml==0.6.0',
    'entrypoints==0.3',
    'et-xmlfile==1.0.1',
    'fsspec==0.4.2',
    'future==0.18.2',
    'graphviz==0.13.2',
    'importlib-metadata==1.4.0',
    'ipykernel==5.1.3',
    'ipython==7.11.1',
    'ipython-genutils==0.2.0',
    'ipywidgets==7.5.1',
    'isort==4.3.21',
    'jdcal==1.4.1',
    'jedi==0.15.2',
    'Jinja2==2.10.3',
    'joblib==0.14.1',
    'jsonschema==3.2.0',
    'jupyter==1.0.0',
    'jupyter-client==5.3.4',
    'jupyter-console==6.0.0',
    'jupyter-core==4.6.1',
    'kiwisolver==1.1.0',
    'lazy-object-proxy==1.4.3',
    'lightgbm==2.3.0',
    'MarkupSafe==1.1.1',
    'matplotlib==3.1.2',
    'mccabe==0.6.1',
    'mistune==0.8.4',
    'more-itertools==8.1.0',
    'nbconvert==5.6.1',
    'nbformat==5.0.3',
    'notebook==5.7.8',
    'numpy==1.17.4',
    'openpyxl==3.0.0',
    'pandas==0.25.3',
    'pandocfilters==1.4.2',
    'paramiko==2.7.1',
    'parso==0.5.2',
    'pexpect==4.7.0',
    'pickleshare==0.7.5',
    'plotly==4.4.1',
    'prometheus-client==0.7.1',
    'prompt-toolkit==2.0.10',
    'ptyprocess==0.6.0',
    'pyarrow==0.14.1',
    'PyContracts==1.8.12',
    'pycparser==2.19',
    'pycryptodome==3.9.4',
    'Pygments==2.5.2',
    'pylint==2.4.4',
    'PyNaCl==1.3.0',
    'pyparsing==2.4.6',
    'pyrsistent==0.15.7',
    'python-dateutil==2.8.1',
    'pytz==2019.3',
    'pyzmq==18.1.1',
    'qtconsole==4.6.0',
    'retrying==1.3.3',
    'scikit-learn==0.22.1',
    'scipy==1.4.1',
    'seaborn==0.9.0',
    'Send2Trash==1.5.0',
    'shap==0.31.0',
    'six==1.14.0',
    'sklearn==0.0',
    'teradata==15.10.0.21',
    'teradatasql==16.20.0.59',
    'terminado==0.8.3',
    'testpath==0.4.4',
    'toolz==0.10.0',
    'tornado==6.0.3',
    'tqdm==4.41.1',
    'traitlets==4.3.3',
    'typed-ast==1.4.1',
    'wcwidth==0.1.8',
    'webencodings==0.5.1',
    'widgetsnbextension==3.5.1',
    'wrapt==1.11.2',
    'xgboost==0.90',
    'XlsxWriter==1.2.6',
    'zipp==1.0.0']

setup(
    name="dspl",
    packages=["dspl"],
    version="0.0.12",
    include_package_data=True,
    install_requires=requirements,
    author="Nikita Varganov",
    author_email="nikita.varganov.ml@gmail.com",
    license="MIT",
    description="Library for using in DS-Platform",
    url="https://github.com/NV-27/dspl",
    download_url="https://github.com/NV-27/dspl/archive/0.0.12.tar.gz",
    keywords=["dspl", "ds_platform", "ds_template"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.6",
        "Operating System :: OS Independent"
    ]
)